﻿#Requires -Version 3.0

#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: AllDemos.ps1
# Description:  
#    The following will be run as part of this demo. Modules can 
#    be skipped by changing the module run flag to FALSE in the 
#    Config.ps1 script.
#               
#   1) Install to setup demo environment (if not previously setup)
#   2) Validation Checks
#   3) Code Analysis
#   4) Execute Script
#   5) Compare Database
#   6) Compare Schemas
#   7) Compare Tables
#   8) Unit Testing (if Code Tester repository is setup)
#-----------------------------------------------------------------

# Set global script name
$global:ScriptName = 'AllDemos'

# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


#-------------------------------------------
# Main Processing
#-------------------------------------------

Show-Log -LogType Txt -LogText  ' '
Show-Log -LogType Txt -LogText  'Running TDT Demo...'
Show-Log -LogType Txt -LogText  ' '


# Run Validation Checks
if ($ValidationCheckRun)
   {& "$PSScriptsDir\ValidationChecks.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Code Analysis
if ($CodeAnalysisRun)
   {& "$PSScriptsDir\CodeAnalysis.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Execute Script
if ($ScriptRun)
   {& "$PSScriptsDir\ExecuteScript.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Compare Databases
if ($CompareDatabasesRun)
   {& "$PSScriptsDir\CompareDatabases.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Schema Compare
if ($CompareSchemasRun)
   {& "$PSScriptsDir\CompareSchemas.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Table Compare
if ($CompareSchemasRun)
   {& "$PSScriptsDir\CompareTables.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Code Tester Check
Show-LogSeperator
$UnitTestingRun = CodeTesterCheck
Show-LogSeperator
Show-Log -LogType Txt -LogText  ' '



# Run Unit Testing based on CT check above
if ($UnitTestingRun)
   {& "$PSScriptsDir\UnitTesting.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Run Debug
if ($DebugRun)
   {& "$PSScriptsDir\Debug.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Display Enums
if ($EnumsRun)
   {& "$PSScriptsDir\ListEnums.ps1"
    Show-Log -LogType Txt -LogText  ' '
   }



# Stop Transcript and Open Log File
Stop-Transcript
Invoke-Item $LogPath
