﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: CodeAnalysis.ps1 
# Description:
#    To demonstrate various Code Analysis calls.           
#-----------------------------------------------------------------


# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Code Analysis Log' 
   $TDT = $null   
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {# Make a Connection
       $NewConnection = $TDT.Connections.NewConnection($CodeAnalysisConnection)
           
       # If Connection Made - set parameters 
       if ($NewConnection) 
          {Show-Log -LogType Txt -LogText 'Setting CodeAnalysis Properties from Config File...'
           $TDT.CodeAnalysis.Connection                          = $NewConnection              
           $TDT.CodeAnalysis.ReportName                          = $CodeAnalysisReportName
           $TDT.CodeAnalysis.RuleSet                             = $CodeAnalysisRuleSet
           $TDT.CodeAnalysis.RuleSetName                         = $CodeAnalysisRuleSetName
           $TDT.CodeAnalysis.OutputFolder                        = $CodeAnalysisOutputFolder
           $TDT.CodeAnalysis.ReportFormats.IncludeHTML           = $CodeAnalysisReportFormatsIncludeHTML
           $TDT.CodeAnalysis.ReportFormats.IncludeJSON           = $CodeAnalysisReportFormatsIncludeJSON
           $TDT.CodeAnalysis.ReportFormats.IncludeXLS            = $CodeAnalysisReportFormatsIncludeXLS
           $TDT.CodeAnalysis.ReportFormats.IncludeXML            = $CodeAnalysisReportFormatsIncludeXML
           $TDT.CodeAnalysis.FailConditions.CheckHalstead        = $CodeAnalysisFailConditionsCheckHalstead        
           $TDT.CodeAnalysis.FailConditions.CheckMaintainability = $CodeAnalysisFailConditionsCheckMaintainability
           $TDT.CodeAnalysis.FailConditions.CheckMcCabe          = $CodeAnalysisFailConditionsCheckMcCabe
           $TDT.CodeAnalysis.FailConditions.CheckRuleViolations  = $CodeAnalysisFailConditionsCheckRuleViolations
           $TDT.CodeAnalysis.FailConditions.CheckSyntaxErrors    = $CodeAnalysisFailConditionsCheckSyntaxErrors
           $TDT.CodeAnalysis.FailConditions.CheckTCR             = $CodeAnalysisFailConditionsCheckTCR
           $TDT.CodeAnalysis.FailConditions.HalsteadLevel        = $CodeAnalysisFailConditionsHalsteadLevel
           $TDT.CodeAnalysis.FailConditions.MaintainabilityLevel = $CodeAnalysisFailConditionsMaintainabilityLevel
           $TDT.CodeAnalysis.FailConditions.McCabeLevel          = $CodeAnalysisFailConditionsMcCabeLevel
           $TDT.CodeAnalysis.FailConditions.TCRLevel             = $CodeAnalysisFailConditionsTCRLevel
           

           # Show Properties
           Show-Log -LogType Txt -LogText "  CodeAnalysis.Connection                          = $CodeAnalysisConnection"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.ReportName                          = $CodeAnalysisReportName"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.RuleSet                             = $CodeAnalysisRuleSet"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.OutputFolder                        = $CodeAnalysisOutputFolder"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FilesFolder                         = $CodeAnalysisInputFolder"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.ReportFormats.IncludeHTML           = $CodeAnalysisReportFormatsIncludeHTML"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.ReportFormats.IncludeJSON           = $CodeAnalysisReportFormatsIncludeJSON"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.ReportFormats.IncludeXLS            = $CodeAnalysisReportFormatsIncludeXLS"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.ReportFormats.IncludeXML            = $CodeAnalysisReportFormatsIncludeXML"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.DBObjects.ObjectName                = $CodeAnalysisDBObjectsObjectName"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.DBObjects.ObjectOwner               = $CodeAnalysisDBObjectsObjectOwner"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.DbObjects.ObjectType                = $CodeAnalysisDBObjectsObjectType"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.DbObjects.ObjectParent              = $CodeAnalysisDBObjectsObjectParent"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.CheckHalstead        = $CodeAnalysisFailConditionsCheckHalstead"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.CheckMaintainability = $CodeAnalysisFailConditionsCheckMaintainability"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.CheckMcCabe          = $CodeAnalysisFailConditionsCheckMcCabe"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.CheckRuleViolations  = $CodeAnalysisFailConditionsCheckRuleViolations"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.CheckSyntaxErrors    = $CodeAnalysisFailConditionsCheckSyntaxErrors"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.CheckTCR             = $CodeAnalysisFailConditionsCheckTCR"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.HalsteadLevel        = $CodeAnalysisFailConditionsHalsteadLevel"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.MaintainabilityLevel = $CodeAnalysisFailConditionsMaintainabilityLevel"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.McCabeLevel          = $CodeAnalysisFailConditionsMcCabeLevel"
           Show-Log -LogType Txt -LogText "  CodeAnalysis.FailConditions.TCRLevel             = $CodeAnalysisFailConditionsTCRLevel"

           Show-LogSeperator

           Show-Log -LogType Txt -LogText "Main Code Analysis Processing..."
            

           # Create a deleteme.txt file that will be added and then deleted later from the set.
           New-Item $CodeAnalysisInputFolder\deleteme.txt -type file -Force -Value 'Dummy file to be deleted' | Out-Null


           # Add each file in $CodeAnalysisInputFolder to CodeAnalysis.Files Object
           $Files = Get-ChildItem $CodeAnalysisInputFolder

           foreach ($CAFile in $Files)
              {Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Add   : $CAFile"
               $TDT.CodeAnalysis.Files.Add("$CodeAnalysisInputFolder\$CAFile")
              }


           # Get File Count
           $CodeAnalysisFileCount = $TDT.CodeAnalysis.Files.Count()
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Count : $CodeAnalysisFileCount"


           # Deleting Dummy File from List and File System - Used for testing purposes
           $TDT.CodeAnalysis.Files.Delete("$CodeAnalysisInputFolder\deleteme.txt")
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Delete: deleteme.txt"
           Remove-Item $CodeAnalysisInputFolder\deleteme.txt | Out-Null


           # Get File Count
           $CodeAnalysisFileCount = $TDT.CodeAnalysis.Files.Count()
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Count : $CodeAnalysisFileCount"


           # List Files added based on Index Number
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Indexof"
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Item  :"
    
           for ($counter=0; $counter -lt $CodeAnalysisFileCount; $counter++) 
               {$FileName   = $TDT.CodeAnalysis.Files.Item($counter)
                $FileIdxPos = $TDT.CodeAnalysis.Files.IndexOf($TDT.CodeAnalysis.Files.Item($counter))
                Show-Log -LogType Txt -LogText "    $FileIdxPos - $Filename"
               }
               

           # Convert File List to a Comma Delimited String - this is just for testing purposes.
           $DelimitedString = $TDT.CodeAnalysis.Files.ToString()
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.ToString"  #$DelimitedString"
 

           # Convert Failed List to a Comma Delimited String - this is just for testing purposes.
           $DelimitedFailesdString = $TDT.CodeAnalysis.FailConditions.ToString()
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.FailConditions.ToString"  #$DelimitedFailedString"              


           # Add Objects, Save JSON, Load JSON (Testing purposes only)
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.DBObjects.Add  : $CodeAnalysisDBObjectsObjectName"
           $DBObject = $TDT.CodeAnalysis.DBObjects.Add()              
           $DBObject.ObjectName      = $CodeAnalysisDBObjectsObjectName
           $DBObject.ObjectOwner     = $CodeAnalysisDBObjectsObjectOwner
           $DBObject.ObjectType      = $CodeAnalysisDBObjectsObjectType
           $DBObject.ObjectParent    = $CodeAnalysisDBObjectsObjectParent

               
           # Get DBObjects Count
           $CodeAnalysisObjectCount = $TDT.CodeAnalysis.DBObjects.Count
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.DBObjects.Count: $CodeAnalysisObjectCount"
                

           # Save DB Objects Properties to JSON File
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.DBObjects.ExportToJSON : $CodeAnalysisDBObjectsPropFile"
           $CAJSONDBObjects = $TDT.CodeAnalysis.DBObjects.ExportToJSON()
           $CAJSONDBObjects | Out-File -FilePath "$CodeAnalysisOutputFolder\$CodeAnalysisDBObjectsPropFile" -Force


           # Save All Properties set above to JSON File
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.ExportToJSON           : $CodeAnalysisAllPropFile"
           $CAJSON = $TDT.CodeAnalysis.ExportToJSON()
           $CAJSON | Out-File -FilePath "$CodeAnalysisOutputFolder\$CodeAnalysisAllPropFile" -Force
           $TDT.CodeAnalysis.DBObjects.Clear()
           $TDT.CodeAnalysis.Files.Clear()                               


           # Load DB Objects Properties from JSON export
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.DBObjects.ImportFromJSON : $CodeAnalysisDBObjectsPropFile"
           $TDT.CodeAnalysis.DBObjects.ImportFromJSON($CAJSONDBObjects)


           # Get DBObjects Count
           $CodeAnalysisObjectCount = $TDT.CodeAnalysis.DBObjects.Count
           $TDT.CodeAnalysis.DBObjects.Clear()
                                 

           # Load All Properties from exported JSON
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.ImportFromJSON            : $CodeAnalysisAllPropFile"
           $TDT.CodeAnalysis.ReportFormats.IncludeXML = $FALSE
           $TDT.CodeAnalysis.ImportFromJSON($CAJSON)


           # Get DBObjects Count
           $CodeAnalysisObjectCount = $TDT.CodeAnalysis.DBObjects.Count
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.DBObjects.Count: $CodeAnalysisObjectCount"

           
           # Get File Count
           $CodeAnalysisFileCount = $TDT.CodeAnalysis.Files.Count()
           Show-Log -LogType Txt -LogText "  Running CodeAnalysis.Files.Count    : $CodeAnalysisFileCount"


           # Execute Code Analysis
           Show-Log -LogType Txt -LogText '  Running CodeAnalysis.Execute'
           $TDT.CodeAnalysis.Execute()

          
           Show-Log -LogType Txt -LogText "Code Analysis Reports Location: $CodeAnalysisOutputFolder"
          

          }
       
       else 
          {Show-Log -LogType Txt -LogText 'Failed to make a connection'}
      
      }
   else
      {Show-Log -LogType Txt -LogText 'Could not start TDT'}


  }

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }

