﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: CompareDatabases.ps1 
# Description:
#    To demonstrate various Compare Database calls.           
#-----------------------------------------------------------------


# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


# Check to see if Install has been run before - if it hasn't, run Install.ps1
if ($SourceDatabase -eq '{SourceDBName}')
   {Show-Log -LogType Txt -LogText 'DEMO schema not found.  Running Install...'
    & "$PSScriptsDir\Install.ps1"
    
    if ($LastExitCode) 
       {exit 1}
    else
       {. (Join-Path $PSScriptRoot '.\Config.ps1')}  #Rerun Config.ps1 after changes were made.
   }


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Compare Database Log'
   $TDT = $null
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {Show-Log -LogType Txt -LogText "Setting Main Parameters from Config File Parameters"
       $TDT.CompareDatabases.DifferenceLimit                            = $CompareDatabasesDifferenceLimit
       $TDT.CompareDatabases.IncludeComments                            = $CompareDatabasesIncludeComments
       $TDT.CompareDatabases.IncludeComments_Dependencies               = $CompareDatabasesIncludeCommentsDependencies
       $TDT.CompareDatabases.InitParams                                 = $CompareDatabasesInitParams
       $TDT.CompareDatabases.IP_BuildAlterSystem                        = $CompareDatabasesIPBuildAlterSystem
       $TDT.CompareDatabases.IP_ExcludeNameType                         = $CompareDatabasesIPExcludeNameType
       $TDT.CompareDatabases.IP_ExcludePathType                         = $CompareDatabasesIPExcludePathType
       $TDT.CompareDatabases.IP_IncludeDefault                          = $CompareDatabasesIPIncludeDefault
       $TDT.CompareDatabases.IP_IncludeUndocumented                     = $CompareDatabasesIPIncludeUndocumented
       $TDT.CompareDatabases.PasswordsNoChange                          = $CompareDatabasesPasswordsNoChange
       $TDT.CompareDatabases.PasswordsNoCompare                         = $CompareDatabasesPasswordsNoCompare
       $TDT.CompareDatabases.ProductComponentVersion                    = $CompareDatabasesProductComponentVersion
       $TDT.CompareDatabases.SafeDrop                                   = $CompareDatabasesSafeDrop
       $TDT.CompareDatabases.SlashEndings                               = $CompareDatabasesSlashEndings
       $TDT.CompareDatabases.VBlockChangeTracking                       = $CompareDatabasesVBlockChangeTracking
       $TDT.CompareDatabases.VControlFile                               = $CompareDatabasesVControlFile
       $TDT.CompareDatabases.VDatabase                                  = $CompareDatabasesVDatabase
       $TDT.CompareDatabases.VInstance                                  = $CompareDatabasesVInstance
       $TDT.CompareDatabases.VObsoleteParameter                         = $CompareDatabasesVObsoleteParameter
       $TDT.CompareDatabases.VObsoleteParameter_Specified               = $CompareDatabasesVObsoleteParameterSpecified
       $TDT.CompareDatabases.VOption                                    = $CompareDatabasesVOption
       $TDT.CompareDatabases.VSGAInfo                                   = $CompareDatabasesVSGAInfo
       $TDT.CompareDatabases.VStatisticsLevel                           = $CompareDatabasesVStatisticsLevel
           
       
       # Set Object Types
       if ($ObjectTypesIncludeAll)
          {Show-Log -LogType Txt -LogText "Setting CompareDatabases.ObjectTypes.IncludeAll()"
           $TDT.CompareDatabases.ObjectTypes.IncludeAll()
          }
        elseif ($ObjectTypesIncludeNone)
          {Show-Log -LogType Txt -LogText "Setting CompareDatabases.ObjectTypes.IncludeNone()"
           $TDT.CompareDatabases.ObjectTypes.IncludeNone()
          }          
        else
          {Show-Log -LogType Txt -LogText "Setting CompareDatabases.ObjectTypes from Config File Parameters"
           $TDT.CompareDatabases.ObjectTypes.AuditPolicies                  = $ObjectTypesAuditPolicies
           $TDT.CompareDatabases.ObjectTypes.AuditsSQLPrivs                 = $ObjectTypesAuditsSQLPrivs
           $TDT.CompareDatabases.ObjectTypes.Audits_All                     = $ObjectTypesAuditsAll
           $TDT.CompareDatabases.ObjectTypes.Audits_Selected                = $ObjectTypesAuditsSelected
           $TDT.CompareDatabases.ObjectTypes.CDBResourcePlans               = $ObjectTypesCDBResourcePlans
           $TDT.CompareDatabases.ObjectTypes.Clustered_Tables               = $ObjectTypesClusteredTables
           $TDT.CompareDatabases.ObjectTypes.Clusters                       = $ObjectTypesClusters
           $TDT.CompareDatabases.ObjectTypes.Constraints                    = $ObjectTypesConstraints
           $TDT.CompareDatabases.ObjectTypes.Constraints_Check              = $ObjectTypesConstraintsCheck
           $TDT.CompareDatabases.ObjectTypes.Constraints_ForeignKey         = $ObjectTypesConstraintsForeignKey
           $TDT.CompareDatabases.ObjectTypes.Constraints_PrimaryKey         = $ObjectTypesConstraintsPrimaryKey
           $TDT.CompareDatabases.ObjectTypes.Constraints_UniqueKey          = $ObjectTypesConstraintsUniqueKey
           $TDT.CompareDatabases.ObjectTypes.Contexts                       = $ObjectTypesContexts
           $TDT.CompareDatabases.ObjectTypes.DBLinks                        = $ObjectTypesDBLinks
           $TDT.CompareDatabases.ObjectTypes.Dimensions                     = $ObjectTypesDimensions
           $TDT.CompareDatabases.ObjectTypes.Directories                    = $ObjectTypesDirectories
           $TDT.CompareDatabases.ObjectTypes.Editions                       = $ObjectTypesEditions
           $TDT.CompareDatabases.ObjectTypes.FlashbackArchives              = $ObjectTypesFlashbackArchives
           $TDT.CompareDatabases.ObjectTypes.Functions                      = $ObjectTypesFunctions
           $TDT.CompareDatabases.ObjectTypes.Grants_All                     = $ObjectTypesGrantsAll
           $TDT.CompareDatabases.ObjectTypes.Grants_Selected                = $ObjectTypesGrantsSelected
           $TDT.CompareDatabases.ObjectTypes.Indexes                        = $ObjectTypesIndexes
           $TDT.CompareDatabases.ObjectTypes.InitParameters                 = $ObjectTypesInitParameters
           $TDT.CompareDatabases.ObjectTypes.Java_Class                     = $ObjectTypesJavaClass
           $TDT.CompareDatabases.ObjectTypes.Java_Resource                  = $ObjectTypesJava_Resource
           $TDT.CompareDatabases.ObjectTypes.Java_Source                    = $ObjectTypesJavaSource
           $TDT.CompareDatabases.ObjectTypes.Jobs                           = $ObjectTypesJobs
           $TDT.CompareDatabases.ObjectTypes.Libraries                      = $ObjectTypesLibraries
           $TDT.CompareDatabases.ObjectTypes.MaterializedViewLogs           = $ObjectTypesMaterializedViewLogs
           $TDT.CompareDatabases.ObjectTypes.MaterializedViews              = $ObjectTypesMaterializedViews
           $TDT.CompareDatabases.ObjectTypes.MaterializedViews_ColComments  = $ObjectTypesMaterializedViewsColComments
           $TDT.CompareDatabases.ObjectTypes.MaterializedViews_Comments     = $ObjectTypesMaterializedViewsComments
           $TDT.CompareDatabases.ObjectTypes.Outlines                       = $ObjectTypesOutlines
           $TDT.CompareDatabases.ObjectTypes.PackageBodies                  = $ObjectTypesPackageBodies
           $TDT.CompareDatabases.ObjectTypes.Packages                       = $ObjectTypesPackages
           $TDT.CompareDatabases.ObjectTypes.Partitions                     = $ObjectTypesPartitions
           $TDT.CompareDatabases.ObjectTypes.Policies                       = $ObjectTypesPolicies                 
           $TDT.CompareDatabases.ObjectTypes.PolicyGroups                   = $ObjectTypesPolicyGroups
           $TDT.CompareDatabases.ObjectTypes.Procedures                     = $ObjectTypesProcedures
           $TDT.CompareDatabases.ObjectTypes.Profiles                       = $ObjectTypesProfiles
           $TDT.CompareDatabases.ObjectTypes.Queues                         = $ObjectTypesQueues
           $TDT.CompareDatabases.ObjectTypes.QueueTables                    = $ObjectTypesQueueTables
           $TDT.CompareDatabases.ObjectTypes.RedactionPolicies              = $ObjectTypesRedactionPolicies
           $TDT.CompareDatabases.ObjectTypes.RedoLogs                       = $ObjectTypesRedoLogs
           $TDT.CompareDatabases.ObjectTypes.RefreshGroups                  = $ObjectTypesRefreshGroups
           $TDT.CompareDatabases.ObjectTypes.ResourceConsumerGroups         = $ObjectTypesResourceConsumerGroups
           $TDT.CompareDatabases.ObjectTypes.ResourcePlans                  = $ObjectTypesResourcePlans
           $TDT.CompareDatabases.ObjectTypes.Roles                          = $ObjectTypesRoles
           $TDT.CompareDatabases.ObjectTypes.RollbackSegments_Private       = $ObjectTypesRollbackSegmentsPrivate
           $TDT.CompareDatabases.ObjectTypes.RollbackSegments_Public        = $ObjectTypesRollbackSegmentsPublic
           $TDT.CompareDatabases.ObjectTypes.SchedulerChains                = $ObjectTypesSchedulerChains
           $TDT.CompareDatabases.ObjectTypes.SchedulerCredentials           = $ObjectTypesSchedulerCredentials
           $TDT.CompareDatabases.ObjectTypes.SchedulerJobClasses            = $ObjectTypesSchedulerJobClasses
           $TDT.CompareDatabases.ObjectTypes.SchedulerJobs                  = $ObjectTypesSchedulerJobs
           $TDT.CompareDatabases.ObjectTypes.SchedulerPrograms              = $ObjectTypesSchedulerPrograms
           $TDT.CompareDatabases.ObjectTypes.SchedulerSchedules             = $ObjectTypesSchedulerSchedules
           $TDT.CompareDatabases.ObjectTypes.SchedulerWindowGroups          = $ObjectTypesSchedulerWindowGroups
           $TDT.CompareDatabases.ObjectTypes.SchedulerWindows               = $ObjectTypesSchedulerWindows
           $TDT.CompareDatabases.ObjectTypes.Sequences                      = $ObjectTypesSequences
           $TDT.CompareDatabases.ObjectTypes.SynonymsPrivate                = $ObjectTypesSynonymsPrivate
           $TDT.CompareDatabases.ObjectTypes.SynonymsPrivateForSchema       = $ObjectTypesSynonymsPrivateForSchema
           $TDT.CompareDatabases.ObjectTypes.SynonymsPublic                 = $ObjectTypesSynonymsPublic
           $TDT.CompareDatabases.ObjectTypes.Tables                         = $ObjectTypesTables
           $TDT.CompareDatabases.ObjectTypes.Tablespaces                    = $ObjectTypesTablespaces
           $TDT.CompareDatabases.ObjectTypes.Tables_ColComments             = $ObjectTypesTablesColComments
           $TDT.CompareDatabases.ObjectTypes.Tables_Comments                = $ObjectTypesTablesComments
           $TDT.CompareDatabases.ObjectTypes.Triggers                       = $ObjectTypesTriggers
           $TDT.CompareDatabases.ObjectTypes.TypeBodies                     = $ObjectTypesTypeBodies
           $TDT.CompareDatabases.ObjectTypes.Types                          = $ObjectTypesTypes
           $TDT.CompareDatabases.ObjectTypes.UnifiedAuditPolicies           = $ObjectTypesUnifiedAuditPolicies
           $TDT.CompareDatabases.ObjectTypes.UnifiedAuditPoliciesEnablement = $ObjectTypesUnifiedAuditPoliciesEnablement
           $TDT.CompareDatabases.ObjectTypes.UserDefaultTablespace          = $ObjectTypesUserDefaultTablespace
           $TDT.CompareDatabases.ObjectTypes.UserProfiles                   = $ObjectTypesUserProfiles
           $TDT.CompareDatabases.ObjectTypes.UserRoles                      = $ObjectTypesUserRoles
           $TDT.CompareDatabases.ObjectTypes.Users                          = $ObjectTypesUsers
           $TDT.CompareDatabases.ObjectTypes.Users_WithObjectsOnly          = $ObjectTypesUsersWithObjectsOnly
           $TDT.CompareDatabases.ObjectTypes.Users_WithoutObjectsOnly       = $ObjectTypesUsersWithoutObjectsOnly
           $TDT.CompareDatabases.ObjectTypes.UserTablespacesHavingObjects   = $ObjectTypesUserTablespacesHavingObjects
           $TDT.CompareDatabases.ObjectTypes.UserTempTablespace             = $ObjectTypesUserTempTablespace
           $TDT.CompareDatabases.ObjectTypes.Views                          = $ObjectTypesViews
           $TDT.CompareDatabases.ObjectTypes.Views_ColComments              = $ObjectTypesViewsColComments
           $TDT.CompareDatabases.ObjectTypes.Views_ColScale                 = $ObjectTypesViewsColScale
           $TDT.CompareDatabases.ObjectTypes.Views_Comments                 = $ObjectTypesViewsComments
           $TDT.CompareDatabases.ObjectTypes.ZoneMaps                       = $ObjectTypesZoneMaps 
          }                      



       #-- Saving Settings to XML file to be loaded in second execute
       $TDT.CompareDatabases.SourceDatabase                = $CompareDatabasesSourceDatabase
       $TDT.CompareDatabases.SourceDatabaseSnapshotComment = $CompareDatabasesSourceDatabaseSnapshotComment     
              
       
       #-- Add Target Information
       $Target = $TDT.CompareDatabases.Targets.Add()
       $Target.Connection            = $CompareDatabasesTargetDatabase
       $Target.DifferenceDetails     = $CompareDatabasesTargetDifferenceDetails
       $Target.DifferenceSummary     = $CompareDatabasesTargetDifferenceSummary
       $Target.SchemaSnapshot        = $CompareDatabasesTargetSchemaSnapshot
       $Target.SyncScript            = $CompareDatabasesTargetSyncScript
       $Target.SchemaSnapshotComment = $CompareDatabasesTargetSchemaSnapshotComment 


       #-- Saving Settings to JSON file to be loaded in second execute
       Show-Log -LogType Txt -LogText "Running CompareDatabases.ExportToJSON(): $CompareDatabasesSourceDatabaseSnapshot"
       $JSON = $TDT.CompareDatabases.ExportToJSON()
       $JSON | Out-File -FilePath "$CompareDatabasesSourceDatabaseSnapshot" -Force
       
             
       #-- Make Connections to Source and Target
       $Source = $TDT.Connections.NewConnection($SourceConnection)
       $Target = $TDT.Connections.NewConnection($TargetConnection)
   

       # Exclude TC_ADMIN_ROLE from Objects to Compare
       Show-Log -LogType Txt -LogText "Running CompareDatabases.ObjectSet.Add: $CompareDatabasesObjectSet"
       $TDT.CompareDatabases.ObjectSet.Add("$CompareDatabasesObjectSet")
       $ObjectSetCount = $TDT.CompareDatabases.ObjectSet.Count
       Show-Log -LogType Txt -LogText "Running CompareDatabases.ObjectSet.Count: $ObjectSetCount"

       
       # Running based on Parameters supplied in Config File.
       Show-Log -LogType Txt -LogText 'Running CompareDatabases.Execute for Parameters Supplied in Config File...'
       $TDT.CompareDatabases.Execute()



       #-- Loading Settings from JSON and running again
       Show-Log -LogType Txt -LogText "Running CompareDatabases.ImportFromJSON()"
       $TDT.CompareDatabases.ImportFromJSON($JSON) 
       Show-Log -LogType Txt -LogText 'Running CompareDatabases.Execute for Parameters Supplied from JSON...'
       $TDT.CompareDatabases.Execute()


       Show-Log -LogType Txt -LogText "DB Compare Report Location: $CompareDatabaseDir\Output"

       
   }

}

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }

