﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: CompareSchemas.ps1 
# Description:
#    To demonstrate various Schema Compare calls.           
#-----------------------------------------------------------------

# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


# Check to see if Install has been run before - if it hasn't, run Install.ps1
if ($SourceDatabase -eq '{SourceDBName}')
   {Show-Log -LogType Txt -LogText 'DEMO schema not found.  Running Install...'
    & "$PSScriptsDir\Install.ps1"
    
    if ($LastExitCode) 
       {exit 1}
    else
       {. (Join-Path $PSScriptRoot '.\Config.ps1')}  #Rerun Config.ps1 after changes were made.
   }


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Compare Schemas Log'
   $TDT = $null
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {Show-Log -LogType Txt -LogText "Setting Main Parameters from Config File Parameters"
       $TDT.CompareSchemas.DifferenceLimit               = $CompareSchemasDifferenceLimit
       $TDT.CompareSchemas.ExclusionFile                 = $CompareSchemasExclusionFile
       $TDT.CompareSchemas.IncludeComments               = $CompareSchemasIncludeComments
       $TDT.CompareSchemas.IncludeComments_Dependencies  = $CompareSchemasIncludeCommentsDependencies
       $TDT.CompareSchemas.IncludeSchemaName             = $CompareSchemasIncludeSchemaName
       $TDT.CompareSchemas.ObjectNameFilterByCase        = $CompareSchemasObjectNameFilterByCase
       $TDT.CompareSchemas.ObjectNameFilterByNOT         = $CompareSchemasObjectNameFilterByNOT
       $TDT.CompareSchemas.ObjectNameFilters             = $CompareSchemasObjectNameFilters
       $TDT.CompareSchemas.PromptComments                = $CompareSchemasPromptComments
       $TDT.CompareSchemas.SetDefineOffCommands          = $CompareSchemasSetDefineOffCommands
       $TDT.CompareSchemas.SlashEndings                  = $CompareSchemasSlashEndings          


       # Set Storage Options
       if ($CompareSchemasStorageOptionsIncludeAll)
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.StorageOptions.IncludeAll()"
           $TDT.CompareSchemas.StorageOptions.IncludeAll()
          }
        elseif ($CompareSchemasStorageOptionsIncludeNone)
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.StorageOptions.IncludeNone()"
           $TDT.CompareSchemas.StorageOptions.IncludeNone()
          }          
        else
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.StorageOptions from Config File Parameters"
           $TDT.CompareSchemas.StorageOptions.BufferPool      = $CompareSchemasStorageOptionsBufferPool
           $TDT.CompareSchemas.StorageOptions.Cache           = $CompareSchemasStorageOptionsCache
           $TDT.CompareSchemas.StorageOptions.Compress        = $CompareSchemasStorageOptionsCompress
           $TDT.CompareSchemas.StorageOptions.Degree          = $CompareSchemasStorageOptionsDegree
           $TDT.CompareSchemas.StorageOptions.FreelistGroups  = $CompareSchemasStorageOptionsFreelistGroups
           $TDT.CompareSchemas.StorageOptions.Freelists       = $CompareSchemasStorageOptionsFreelists
           $TDT.CompareSchemas.StorageOptions.Initial         = $CompareSchemasStorageOptionsInitial
           $TDT.CompareSchemas.StorageOptions.IniTrans        = $CompareSchemasStorageOptionsIniTrans
           $TDT.CompareSchemas.StorageOptions.Instances       = $CompareSchemasStorageOptionsInstances
           $TDT.CompareSchemas.StorageOptions.LobStorage      = $CompareSchemasStorageOptionsLobStorage
           $TDT.CompareSchemas.StorageOptions.Logging         = $CompareSchemasStorageOptionsLogging
           $TDT.CompareSchemas.StorageOptions.MaxExtents      = $CompareSchemasStorageOptionsMaxExtents
           $TDT.CompareSchemas.StorageOptions.MaxSize         = $CompareSchemasStorageOptionsMaxSize
           $TDT.CompareSchemas.StorageOptions.MaxTrans        = $CompareSchemasStorageOptionsMaxTrans
           $TDT.CompareSchemas.StorageOptions.MinExtents      = $CompareSchemasStorageOptionsMinExtents
           $TDT.CompareSchemas.StorageOptions.Monitoring      = $CompareSchemasStorageOptionsMonitoring
           $TDT.CompareSchemas.StorageOptions.Next            = $CompareSchemasStorageOptionsNext
           $TDT.CompareSchemas.StorageOptions.PctFree         = $CompareSchemasStorageOptionsPctFree
           $TDT.CompareSchemas.StorageOptions.PctIncrease     = $CompareSchemasStorageOptionsPctIncrease
           $TDT.CompareSchemas.StorageOptions.PctUsed         = $CompareSchemasStorageOptionsPctUsed
           $TDT.CompareSchemas.StorageOptions.RowMovement     = $CompareSchemasStorageOptionsRowMovement
           $TDT.CompareSchemas.StorageOptions.Tablespace      = $CompareSchemasStorageOptionsTablespace
          }


       # Set Type Options
       if ($CompareSchemasTypeOptionsIncludeAll)
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.TypeOptions.IncludeAll()"
           $TDT.CompareSchemas.TypeOptions.IncludeAll()
          }
        elseif ($CompareSchemasTypeOptionsIncludeNone)
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.TypeOptions.IncludeNone()"
           $TDT.CompareSchemas.TypeOptions.IncludeNone()
          }          
        else
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.TypeOptions from Config File Parameters"
           $TDT.CompareSchemas.TypeOptions.ColumnBYTECHAR            = $CompareSchemasTypeOptionsColumnBYTECHAR
           $TDT.CompareSchemas.TypeOptions.ColumnScale               = $CompareSchemasTypeOptionsColumnScale
           $TDT.CompareSchemas.TypeOptions.ConstraintCompareRef      = $CompareSchemasTypeOptionsConstraintCompareRef
           $TDT.CompareSchemas.TypeOptions.ConstraintIgnoreNameDiffs = $CompareSchemasTypeOptionsConstraintIgnoreNameDiffs
           $TDT.CompareSchemas.TypeOptions.GrantRevokeREFS           = $CompareSchemasTypeOptionsGrantRevokeREFS
           $TDT.CompareSchemas.TypeOptions.ObjectStatus              = $CompareSchemasTypeOptionsObjectStatus
           $TDT.CompareSchemas.TypeOptions.SequencesCurrentVal       = $CompareSchemasTypeOptionsSequencesCurrentVal
           $TDT.CompareSchemas.TypeOptions.SequencesDrop             = $CompareSchemasTypeOptionsSequencesDrop
           $TDT.CompareSchemas.TypeOptions.SequencesMinVal           = $CompareSchemasTypeOptionsSequencesMinVal
           $TDT.CompareSchemas.TypeOptions.SourceCaseSensitive       = $CompareSchemasTypeOptionsSourceCaseSensitive
           $TDT.CompareSchemas.TypeOptions.SourceExcludeComments     = $CompareSchemasTypeOptionsSourceExcludeComments
           $TDT.CompareSchemas.TypeOptions.SourceExcludeQuotes       = $CompareSchemasTypeOptionsSourceExcludeQuotes
           $TDT.CompareSchemas.TypeOptions.SourceExcludeWhitespace   = $CompareSchemasTypeOptionsSourceExcludeWhitespace
           $TDT.CompareSchemas.TypeOptions.SourceFormatFirst         = $CompareSchemasTypeOptionsSourceFormatFirst
           $TDT.CompareSchemas.TypeOptions.SourceListStats           = $CompareSchemasTypeOptionsSourceListStats
           $TDT.CompareSchemas.TypeOptions.SynonymsTarget            = $CompareSchemasTypeOptionsSynonymsTarget
           $TDT.CompareSchemas.TypeOptions.TablesAnalyzed            = $CompareSchemasTypeOptionsTablesAnalyzed
           $TDT.CompareSchemas.TypeOptions.TablesBlankNULLs          = $CompareSchemasTypeOptionsTablesBlankNULLs
           $TDT.CompareSchemas.TypeOptions.TablesColumnOrder         = $CompareSchemasTypeOptionsTablesColumnOrder
           $TDT.CompareSchemas.TypeOptions.TablesLogging             = $CompareSchemasTypeOptionsTablesLogging
           $TDT.CompareSchemas.TypeOptions.TablesNUMROWS             = $CompareSchemasTypeOptionsTablesNUMROWS
           $TDT.CompareSchemas.TypeOptions.TablesSelectCount         = $CompareSchemasTypeOptionsTablesSelectCount
           $TDT.CompareSchemas.TypeOptions.TablesUseDBMSRedef        = $CompareSchemasTypeOptionsTablesUseDBMSRedef
           $TDT.CompareSchemas.TypeOptions.TriggersParse             = $CompareSchemasTypeOptionsTriggersParse
           $TDT.CompareSchemas.TypeOptions.TriggersSchemaDesc        = $CompareSchemasTypeOptionsTriggersSchemaDesc
          }

 
       # Set Object Types
       if ($ObjectTypesIncludeAll)
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.ObjectTypes.IncludeAll()"
           $TDT.CompareSchemas.ObjectTypes.IncludeAll()
          }
        elseif ($ObjectTypesIncludeNone)
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.ObjectTypes.IncludeNone()"
           $TDT.CompareSchemas.ObjectTypes.IncludeNone()
          }          
        else
          {Show-Log -LogType Txt -LogText "Setting CompareSchemas.ObjectTypes from Config File Parameters"
           $TDT.CompareSchemas.ObjectTypes.AuditPolicies                  = $ObjectTypesAuditPolicies
           $TDT.CompareSchemas.ObjectTypes.AuditsSQLPrivs                 = $ObjectTypesAuditsSQLPrivs
           $TDT.CompareSchemas.ObjectTypes.Audits_All                     = $ObjectTypesAuditsAll
           $TDT.CompareSchemas.ObjectTypes.Audits_Selected                = $ObjectTypesAuditsSelected
           $TDT.CompareSchemas.ObjectTypes.CDBResourcePlans               = $ObjectTypesCDBResourcePlans
           $TDT.CompareSchemas.ObjectTypes.Clustered_Tables               = $ObjectTypesClusteredTables
           $TDT.CompareSchemas.ObjectTypes.Clusters                       = $ObjectTypesClusters
           $TDT.CompareSchemas.ObjectTypes.Constraints                    = $ObjectTypesConstraints
           $TDT.CompareSchemas.ObjectTypes.Constraints_Check              = $ObjectTypesConstraintsCheck
           $TDT.CompareSchemas.ObjectTypes.Constraints_ForeignKey         = $ObjectTypesConstraintsForeignKey
           $TDT.CompareSchemas.ObjectTypes.Constraints_PrimaryKey         = $ObjectTypesConstraintsPrimaryKey
           $TDT.CompareSchemas.ObjectTypes.Constraints_UniqueKey          = $ObjectTypesConstraintsUniqueKey
           $TDT.CompareSchemas.ObjectTypes.Contexts                       = $ObjectTypesContexts
           $TDT.CompareSchemas.ObjectTypes.DBLinks                        = $ObjectTypesDBLinks
           $TDT.CompareSchemas.ObjectTypes.Dimensions                     = $ObjectTypesDimensions
           $TDT.CompareSchemas.ObjectTypes.Directories                    = $ObjectTypesDirectories
           $TDT.CompareSchemas.ObjectTypes.Editions                       = $ObjectTypesEditions
           $TDT.CompareSchemas.ObjectTypes.FlashbackArchives              = $ObjectTypesFlashbackArchives
           $TDT.CompareSchemas.ObjectTypes.Functions                      = $ObjectTypesFunctions
           $TDT.CompareSchemas.ObjectTypes.Grants_All                     = $ObjectTypesGrantsAll
           $TDT.CompareSchemas.ObjectTypes.Grants_Selected                = $ObjectTypesGrantsSelected
           $TDT.CompareSchemas.ObjectTypes.Indexes                        = $ObjectTypesIndexes
           $TDT.CompareSchemas.ObjectTypes.InitParameters                 = $ObjectTypesInitParameters
           $TDT.CompareSchemas.ObjectTypes.Java_Class                     = $ObjectTypesJavaClass
           $TDT.CompareSchemas.ObjectTypes.Java_Resource                  = $ObjectTypesJava_Resource
           $TDT.CompareSchemas.ObjectTypes.Java_Source                    = $ObjectTypesJavaSource
           $TDT.CompareSchemas.ObjectTypes.Jobs                           = $ObjectTypesJobs
           $TDT.CompareSchemas.ObjectTypes.Libraries                      = $ObjectTypesLibraries
           $TDT.CompareSchemas.ObjectTypes.MaterializedViewLogs           = $ObjectTypesMaterializedViewLogs
           $TDT.CompareSchemas.ObjectTypes.MaterializedViews              = $ObjectTypesMaterializedViews
           $TDT.CompareSchemas.ObjectTypes.MaterializedViews_ColComments  = $ObjectTypesMaterializedViewsColComments
           $TDT.CompareSchemas.ObjectTypes.MaterializedViews_Comments     = $ObjectTypesMaterializedViewsComments
           $TDT.CompareSchemas.ObjectTypes.Outlines                       = $ObjectTypesOutlines
           $TDT.CompareSchemas.ObjectTypes.PackageBodies                  = $ObjectTypesPackageBodies
           $TDT.CompareSchemas.ObjectTypes.Packages                       = $ObjectTypesPackages
           $TDT.CompareSchemas.ObjectTypes.Partitions                     = $ObjectTypesPartitions
           $TDT.CompareSchemas.ObjectTypes.Policies                       = $ObjectTypesPolicies                 
           $TDT.CompareSchemas.ObjectTypes.PolicyGroups                   = $ObjectTypesPolicyGroups
           $TDT.CompareSchemas.ObjectTypes.Procedures                     = $ObjectTypesProcedures
           $TDT.CompareSchemas.ObjectTypes.Profiles                       = $ObjectTypesProfiles
           $TDT.CompareSchemas.ObjectTypes.Queues                         = $ObjectTypesQueues
           $TDT.CompareSchemas.ObjectTypes.QueueTables                    = $ObjectTypesQueueTables
           $TDT.CompareSchemas.ObjectTypes.RedactionPolicies              = $ObjectTypesRedactionPolicies
           $TDT.CompareSchemas.ObjectTypes.RedoLogs                       = $ObjectTypesRedoLogs
           $TDT.CompareSchemas.ObjectTypes.RefreshGroups                  = $ObjectTypesRefreshGroups
           $TDT.CompareSchemas.ObjectTypes.ResourceConsumerGroups         = $ObjectTypesResourceConsumerGroups
           $TDT.CompareSchemas.ObjectTypes.ResourcePlans                  = $ObjectTypesResourcePlans
           $TDT.CompareSchemas.ObjectTypes.Roles                          = $ObjectTypesRoles
           $TDT.CompareSchemas.ObjectTypes.RollbackSegments_Private       = $ObjectTypesRollbackSegmentsPrivate
           $TDT.CompareSchemas.ObjectTypes.RollbackSegments_Public        = $ObjectTypesRollbackSegmentsPublic
           $TDT.CompareSchemas.ObjectTypes.SchedulerChains                = $ObjectTypesSchedulerChains
           $TDT.CompareSchemas.ObjectTypes.SchedulerCredentials           = $ObjectTypesSchedulerCredentials
           $TDT.CompareSchemas.ObjectTypes.SchedulerJobClasses            = $ObjectTypesSchedulerJobClasses
           $TDT.CompareSchemas.ObjectTypes.SchedulerJobs                  = $ObjectTypesSchedulerJobs
           $TDT.CompareSchemas.ObjectTypes.SchedulerPrograms              = $ObjectTypesSchedulerPrograms
           $TDT.CompareSchemas.ObjectTypes.SchedulerSchedules             = $ObjectTypesSchedulerSchedules
           $TDT.CompareSchemas.ObjectTypes.SchedulerWindowGroups          = $ObjectTypesSchedulerWindowGroups
           $TDT.CompareSchemas.ObjectTypes.SchedulerWindows               = $ObjectTypesSchedulerWindows
           $TDT.CompareSchemas.ObjectTypes.Sequences                      = $ObjectTypesSequences
           $TDT.CompareSchemas.ObjectTypes.SynonymsPrivate                = $ObjectTypesSynonymsPrivate
           $TDT.CompareSchemas.ObjectTypes.SynonymsPrivateForSchema       = $ObjectTypesSynonymsPrivateForSchema
           $TDT.CompareSchemas.ObjectTypes.SynonymsPublic                 = $ObjectTypesSynonymsPublic
           $TDT.CompareSchemas.ObjectTypes.Tables                         = $ObjectTypesTables
           $TDT.CompareSchemas.ObjectTypes.Tablespaces                    = $ObjectTypesTablespaces
           $TDT.CompareSchemas.ObjectTypes.Tables_ColComments             = $ObjectTypesTablesColComments
           $TDT.CompareSchemas.ObjectTypes.Tables_Comments                = $ObjectTypesTablesComments
           $TDT.CompareSchemas.ObjectTypes.Triggers                       = $ObjectTypesTriggers
           $TDT.CompareSchemas.ObjectTypes.TypeBodies                     = $ObjectTypesTypeBodies
           $TDT.CompareSchemas.ObjectTypes.Types                          = $ObjectTypesTypes
           $TDT.CompareSchemas.ObjectTypes.UnifiedAuditPolicies           = $ObjectTypesUnifiedAuditPolicies
           $TDT.CompareSchemas.ObjectTypes.UnifiedAuditPoliciesEnablement = $ObjectTypesUnifiedAuditPoliciesEnablement
           $TDT.CompareSchemas.ObjectTypes.UserDefaultTablespace          = $ObjectTypesUserDefaultTablespace
           $TDT.CompareSchemas.ObjectTypes.UserProfiles                   = $ObjectTypesUserProfiles
           $TDT.CompareSchemas.ObjectTypes.UserRoles                      = $ObjectTypesUserRoles
           $TDT.CompareSchemas.ObjectTypes.Users                          = $ObjectTypesUsers
           $TDT.CompareSchemas.ObjectTypes.Users_WithObjectsOnly          = $ObjectTypesUsersWithObjectsOnly
           $TDT.CompareSchemas.ObjectTypes.Users_WithoutObjectsOnly       = $ObjectTypesUsersWithoutObjectsOnly
           $TDT.CompareSchemas.ObjectTypes.UserTablespacesHavingObjects   = $ObjectTypesUserTablespacesHavingObjects
           $TDT.CompareSchemas.ObjectTypes.UserTempTablespace             = $ObjectTypesUserTempTablespace
           $TDT.CompareSchemas.ObjectTypes.Views                          = $ObjectTypesViews
           $TDT.CompareSchemas.ObjectTypes.Views_ColComments              = $ObjectTypesViewsColComments
           $TDT.CompareSchemas.ObjectTypes.Views_ColScale                 = $ObjectTypesViewsColScale
           $TDT.CompareSchemas.ObjectTypes.Views_Comments                 = $ObjectTypesViewsComments
           $TDT.CompareSchemas.ObjectTypes.ZoneMaps                       = $ObjectTypesZoneMaps 
          }                      


       #-- Make Connections to Source and Target
       $Source = $TDT.Connections.NewConnection($SourceConnection)
       $Target = $TDT.Connections.NewConnection($TargetConnection)

       
       #-- Setting Source Connection Information
       $TDT.CompareSchemas.Source.Connection             = $Source      
       $TDT.CompareSchemas.Source.Schema                 = $SourceSchema
       $TDT.CompareSchemas.Source.SnapshotOutputComment  = $CompareSchemasSourceSchemaSnapshotComment     

       
       #-- Setting Target Connection Information
       $TDT.CompareSchemas.Target.Connection             = $Target
       $TDT.CompareSchemas.Target.Schema                 = $TargetSchema
       $TDT.CompareSchemas.Target.SnapshotOutputComment  = $CompareSchemasTargetSchemaSnapshot             
       

       #-- Saving Settings to JSON file to be loaded in second execute
       Show-Log -LogType Txt -LogText "Running CompareSchemas.ExportToJSON(): $CompareSchemasSourceSchemasnapshot"
       $JSON = $TDT.CompareSchemas.ExportToJSON()
       $JSON | Out-File -FilePath "$CompareSchemasSourceSchemasnapshot" -Force
        

       # Exclude TC_ADMIN_ROLE from Objects to Compare
       Show-Log -LogType Txt -LogText "Running CompareSchemas.ObjectSet.Add: $CompareSchemasObjectSet"
       $TDT.CompareSchemas.ObjectSet.Add("$CompareSchemasObjectSet")
       $ObjectSetCount = $TDT.CompareSchemas.ObjectSet.Count
       Show-Log -LogType Txt -LogText "Running CompareSchemas.ObjectSet.Count: $ObjectSetCount"

       
       # Running based on Parameters supplied in Config File.
       Show-Log -LogType Txt -LogText 'Running CompareSchemas.Execute for Parameters Supplied in Config File...'
       $TDT.CompareSchemas.Execute()
     

       #-- Loading Settings to JSON and running again
       Show-Log -LogType Txt -LogText "Running CompareSchemas.ImportFromJSON()"
       $TDT.CompareSchemas.ImportFromJSON($JSON) 
       Show-Log -LogType Txt -LogText 'Running CompareSchemas.Execute for Parameters Supplied from JSON...'


       #  Execute Schema Compare
       $TDT.CompareSchemas.Execute()
       
       
       # Save schema comparison script to file
       Show-Log -LogType Txt -LogText 'Running CompareSchemas.GetScript'
       $Script = $TDT.CompareSchemas.GetScript()
       $Script | Out-File -FilePath $CompareSchemasScript -Force
  
  
       # Save CSV difference details to file
       Show-Log -LogType Txt -LogText 'Running CompareSchemas.GetDifferenceDetailsCSV'
       $DifferenceDetailsCSV = $TDT.CompareSchemas.GetDifferenceDetailsCSV()
       $DifferenceDetailsCSV | Out-File -FilePath $CompareSchemasDifferenceDetailsCSV -Force
  
  
       # Save HTML difference details to file
       Show-Log -LogType Txt -LogText 'Running CompareSchemas.GetDifferenceDetailsHTML'
       $DifferenceDetailsHTML = $TDT.CompareSchemas.GetDifferenceDetailsHTML()
       $DifferenceDetailsHTML | Out-File -FilePath $CompareSchemasDifferenceDetailsHTML -Force
  
  
       # Save TXT difference details to file
       Show-Log -LogType Txt -LogText 'Running CompareSchemas.GetDifferenceDetailsTXT'
       $DifferenceDetailsTXT = $TDT.CompareSchemas.GetDifferenceDetailsTXT()
       $DifferenceDetailsTXT | Out-File -FilePath $CompareSchemasDifferenceDetailsTXT -Force

       Show-Log -LogType Txt -LogText "Schema Compare Report Location: $CompareSchemaDir\Output"
 
           
   }

}

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }

