﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: CompareTables.ps1 
# Description:
#    To demonstrate various Table Compare calls.           
#-----------------------------------------------------------------

# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


# Check to see if Install has been run before - if it hasn't, run Install.ps1
if ($SourceDatabase -eq '{SourceDBName}')
   {Show-Log -LogType Txt -LogText 'DEMO schema not found.  Running Install...'
    & "$PSScriptsDir\Install.ps1"
    
    if ($LastExitCode) 
       {exit 1}
    else
       {. (Join-Path $PSScriptRoot '.\Config.ps1')}  #Rerun Config.ps1 after changes were made.
   }


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Compare Tables Log'
   $TDT = $null
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {Show-Log -LogType Txt -LogText "Setting Main Parameters from Config File Parameters"
       $TDT.CompareMultipleTables.ArrayDML                = $CompareMultipleTablesArrayDML
       $TDT.CompareMultipleTables.ColumnMatchMode         = $CompareMultipleTablesColumnMatchMode
       $TDT.CompareMultipleTables.CreateDBLink            = $CompareMultipleTablesCreateDBLink
       $TDT.CompareMultipleTables.DBLinkName              = $CompareMultipleTablesDBLinkName
       $TDT.CompareMultipleTables.DisableTriggers         = $CompareMultipleTablesDisableTriggers
       $TDT.CompareMultipleTables.EnableTriggers          = $CompareMultipleTablesEnableTriggers
       $TDT.CompareMultipleTables.ErrorMode               = $CompareMultipleTablesErrorMode 
       $TDT.CompareMultipleTables.SyncFile                = $CompareMultipleTablesSyncFile
       $TDT.CompareMultipleTables.SyncFolder              = $CompareMultipleTablesSyncFolder
       $TDT.CompareMultipleTables.SyncMode                = $CompareMultipleTablesSyncMode
       $TDT.CompareMultipleTables.TableAppend             = $CompareMultipleTablesTableAppend
       $TDT.CompareMultipleTables.TableFilter             = $CompareMultipleTablesTableFilter
       $TDT.CompareMultipleTables.TableMatchMode          = $CompareMultipleTablesTableMatchMode 
       $TDT.CompareMultipleTables.TablePrepend            = $CompareMultipleTablesTablePrepend
       $TDT.CompareMultipleTables.TableReplaceWith        = $CompareMultipleTablesTableReplaceWith
       $TDT.CompareMultipleTables.TableSearchFor          = $CompareMultipleTablesTableSearchFor
       $TDT.CompareMultipleTables.UnsupportedDatatypeMode = $CompareMultipleTablesUnsupportedDatatypeMode          


      
       #-- Make Connections to Source and Target
       $Source = $TDT.Connections.NewConnection($SourceConnection)
       $Target = $TDT.Connections.NewConnection($TargetConnection)

       
       #-- Set Source Connection/Schema Information
       $TDT.CompareMultipleTables.SourceConnection       = $CompareMultipleTablesSourceConnection
       $TDT.CompareMultipleTables.SourceSchema           = $CompareMultipleTablesSourceSchema   


       #-- Set Target Connection/Schema Information
       $TDT.CompareMultipleTables.TargetConnection       = $CompareMultipleTablesTargetConnection
       $TDT.CompareMultipleTables.TargetSchema           = $CompareMultipleTablesTargetSchema          


       #-- Saving Settings to JSON file to be loaded in second execute
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.ExportToJSON(): $CompareMultipleTablesSourceSnapshot"
       $JSON = $TDT.CompareMultipleTables.ExportToJSON()
       $JSON | Out-File -FilePath "$CompareMultipleTablesSourceSnapshot" -Force
 


       #-- Loading Settings from JSON to be loaded
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.ImportFromJSON()"
       $TDT.CompareMultipleTables.ImportFromJSON($JSON)       


      
       # Running based on Parameters supplied in Config File.
       Show-Log -LogType Txt -LogText 'Running CompareMultipleTables.Execute for Parameters Supplied in Config File...'
       $TDT.CompareMultipleTables.Execute()


       # Add Tables Mappings from parameters in Config file
       Show-Log -LogType Txt -LogText 'Running CompareMultipleTables.TableMappings.ExportToJSON() for Parameters Supplied in Config File...'
       $TDT.CompareMultipleTables.TableMappings.Add($CompareMultipleTablesTableMappingsTable)
       

       # Export JSON to File
       $JSON = $TDT.CompareMultipleTables.TableMappings.ExportToJSON()
       $JSON | Out-File -FilePath "$CompareMultipleTablesTableMappingsFile" -Force
       

       # Import JSON from Config File Parameters
       Show-Log -LogType Txt -LogText 'Running CompareMultipleTables.TableMappings.ImportFromJSON() for Parameters Supplied in Config File...'
       $TDT.CompareMultipleTables.TableMappings.ImportFromJSON($JSON)


       # Get TableMapping Count
       $TableMappingsCount  = $TDT.CompareMultipleTables.TableMappings.Count()
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.TableMappings.Count: $TableMappingsCount"


       # List Files added based on Index Number
       Show-Log -LogType Txt -LogText "  Running CompareMultipleTables.TableMappings.Item"
       Show-Log -LogType Txt -LogText "  Running CompareMultipleTables.TableMappings.IndexOf:"
            
       for ($counter=0; $counter -lt $TableMappingsCount; $counter++) 
           {$IndxName   = $TDT.CompareMultipleTables.TableMappings.Item($counter)
            $IndxPos    = $TDT.CompareMultipleTables.TableMappings.IndexOf($TDT.CompareMultipleTables.TableMappings.Item($counter))
            Show-Log -LogType Txt -LogText "    $IndxPos - $IndxName"
           }


       #  Convert Table Mapping to a string
       $TableMappingsString = $TDT.CompareMultipleTables.TableMappings.ToString()
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.TableMappings.ToString(): $TableMappingsString"

       
       #  Delete Table Mapping
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.TableMappings.Delete: $IndxName"
       $TDT.CompareMultipleTables.TableMappings.Delete($IndxName)
       
       
       #  Clear Table Mapping
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.TableMappings.Clear()"
       $TDT.CompareMultipleTables.TableMappings.Clear()
       

       #  Get Table Mapping Count
       $TableMappingsCount  = $TDT.CompareMultipleTables.TableMappings.Count()
       Show-Log -LogType Txt -LogText "Running CompareMultipleTables.TableMappings.Count: $TableMappingsCount"


       Show-Log -LogType Txt -LogText "Table Compare Report Location: $CompareTablesDir\Output"
      
   }


}

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }

