﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: Debug.ps1 
# Description:
#    To demonstrate various Debug calls.           
#-----------------------------------------------------------------


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Debug Log'
   $TDT = $null
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {# Get Logfile Text
       Show-Log -LogType Txt -LogText "Running Debug.Logfile.GetText()"
       $TDT.Debug.Logfile.GetText() | Out-File -FilePath "$DebugDir\Logfile.txt" -Force
      
       # Get Logfile Filename
       $DebugFile = $TDT.Debug.Logfile.Filename()
       Show-Log -LogType Txt -LogText "Running Debug.Logfile.Filename(): $DebugFile"
      
       # Get Exception Log Filename
       $ExceptionLogFile = $TDT.Debug.ExceptionLog.Filename()
       Show-Log -LogType Txt -LogText "Running Debug.ExceptionLog.Filename(): $ExceptionLogFile"

       # Get Exception Log - Last Error
       Show-Log -LogType Txt -LogText "Running Debug.ExceptionLog.GetLastError()"
       $TDT.Debug.ExceptionLog.GetLastError()     | Out-File -FilePath "$DebugDir\LastError.txt" -Force

       # Get Exception Log - Last Call Stack      
       Show-Log -LogType Txt -LogText "Running Debug.ExceptionLog.GetLastCallStack()"
       $TDT.Debug.ExceptionLog.GetLastCallStack() | Out-File -FilePath "$DebugDir\LastCallStack.txt" -Force

       # Get Exception Log - Last Call Last Report            
       Show-Log -LogType Txt -LogText "Running Debug.ExceptionLog.GetLastReport()"
       $TDT.Debug.ExceptionLog.GetLastReport()    | Out-File -FilePath "$DebugDir\LastReport.txt" -Force

       # Get Exception Log - Full Report                  
       Show-Log -LogType Txt -LogText "Running Debug.ExceptionLog.GetFullReport()"
       $TDT.Debug.ExceptionLog.GetFullReport()    | Out-File -FilePath "$DebugDir\FullReport.txt" -Force
      
       Show-LogSeperator
       Show-Log -LogType Txt -LogText "Debug Report Location: $DebugDir\Output"
      
      
      }

  }

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }

