﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: ExecuteScript.ps1 
# Description:
#    To demonstrate various Execute Script calls.           
#-----------------------------------------------------------------

# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


# Check to see if Install has been run before - if it hasn't, run Install.ps1
if ($SourceDatabase -eq '{SourceDBName}')
   {Show-Log -LogType Txt -LogText 'DEMO schema not found.  Running Install...'
    & "$PSScriptsDir\Install.ps1"
    
    if ($LastExitCode) 
       {exit 1}
    else
       {. (Join-Path $PSScriptRoot '.\Config.ps1')}  #Rerun Config.ps1 after changes were made.
   }


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Execute Script Log'
   $TDT = $null
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {# Make a Connection
       $NewConnection = $TDT.Connections.NewConnection($ScriptConnection)

       # If Connection Made - set parameters 
       if ($NewConnection) 
          {Show-Log -LogType Txt -LogText 'Setting Script Execute Properties from Config File...'
           $ScriptConnection                     = $TargetConnection
           $ScriptInputFolder                    = "$ScriptExecuteDir\Input" 
           $ScriptOutputFolder                   = "$ScriptExecuteDir\Output"
           $ScriptMaxRows                        = $ScriptMaxRows
           $ScriptIncludeOutput                  = $TRUE
           $ScriptSelectStmt                     = $ScriptSelectStmt
           
           
           # Show Properties
           Show-Log -LogType Txt -LogText "  Script.Connection    = $ScriptConnection"
           Show-Log -LogType Txt -LogText "  ScriptMaxRows        = $ScriptMaxRows"
           Show-Log -LogType Txt -LogText "  ScriptIncludeOutput  = $ScriptIncludeOutput"
           Show-Log -LogType Txt -LogText "  ScriptInputFolder    = $ScriptInputFolder"
           Show-Log -LogType Txt -LogText "  ScriptOutputFolder   = $ScriptOutputFolder"
           Show-Log -LogType Txt -LogText "  ScriptSelectStmt     = $ScriptSelectStmt"
           Show-LogSeperator
 
                    
          
           # Create a deleteme.txt file that will be added and then deleted later from the set.
           New-Item $ScriptInputFolder\deleteme.txt -type file -Force -Value 'Dummy file to be deleted' | Out-Null

               
           # Get File Count
           $ScriptsCount = $TDT.Scripts.Count()
           Show-Log -LogType Txt -LogText "  Running Scripts.Count: $ScriptsCount"

               
           # Add each file in $ScriptInputFolder
           $Files = Get-ChildItem $ScriptInputFolder

           foreach ($File in $Files)
              {Show-LogSeperator
               Show-Log -LogType Txt -LogText "  Running Scripts.Add         : $File"
               $Script               = $TDT.Scripts.Add()
               $Script.Connection    = $NewConnection
               $Script.IncludeOutput = $ScriptIncludeOutput
               $Script.MaxRows       = $ScriptMaxRows
               $Script.InputFile     = "$ScriptInputFolder\$File"
               $Script.OutputFile    = "$ScriptOutputFolder\$File.out.txt"
               Show-Log -LogType Txt -LogText "  Running Script.Execute      : $ScriptInputFolder\$File in $ScriptConnection"
               $Script.Execute()
               $ScriptsCount         = $TDT.Scripts.Count()
               Show-Log -LogType Txt -LogText "  Running Scripts.Count       : $ScriptsCount"
               $ScriptErrorCount     = $Script.ErrorCount
               Show-Log -LogType Txt -LogText "  Running Script.ErrorCount   : $ScriptErrorCount"
               $ScriptExecutionTime  = $Script.ExecutionTime
               Show-Log -LogType Txt -LogText "  Running Script.ExecutionTime: $ScriptExecutionTime"
               $ScriptOutputText     = $Script.OutputText
               Show-Log -LogType Txt -LogText "  Running Script.OutputText   : $ScriptOutputText"
              }


           Show-LogSeperator

           # List Files added based on Index Number
           Show-Log -LogType Txt -LogText "  Running Scripts.Indexof"
           Show-Log -LogType Txt -LogText "  Running Scripts.Item:"
            
           for ($counter=0; $counter -lt $ScriptsCount; $counter++) 
               {$FileName   = $TDT.Scripts.Item($counter)
                $FileIdxPos = $TDT.Scripts.IndexOf($TDT.Scripts.Item($counter))
                Show-Log -LogType Txt -LogText "    $FileIdxPos - $Filename"
               }
               
         
           # Deleting Dummy File from List and File System - Used for testing purposes
           $TDT.Scripts.Delete(0)
           Show-Log -LogType Txt -LogText "  Running Script.Delete: deleteme.txt"
           Remove-Item $ScriptInputFolder\deleteme.txt | Out-Null
           
           
           # Get File Count
           $ScriptsCount = $TDT.Scripts.Count()
           Show-Log -LogType Txt -LogText "  Running Scripts.Count: $ScriptsCount"
 
           
           # Remove all Scripts from the collection
           Show-Log -LogType Txt -LogText "  Running Scripts.Clear"
           $TDT.Scripts.Clear()
           
           
           # Get File Count
           $ScriptsCount = $TDT.Scripts.Count()
           Show-Log -LogType Txt -LogText "  Running Scripts.Count: $ScriptsCount"

           
           # Add a select statement to the collection and execute
           Show-LogSeperator
           Show-Log -LogType Txt -LogText "  Running Scripts.Add"
           $Script               = $TDT.Scripts.Add()
           $Script.Connection    = $NewConnection
           $Script.IncludeOutput = $ScriptIncludeOutput
           $Script.MaxRows       = $ScriptMaxRows
           $Script.InputText     = $ScriptSelectStmt
           Show-Log -LogType Txt -LogText "  Running Script.Execute      : $ScriptSelectStmt in $ScriptConnection"
           $Script.Execute()
           $ScriptsCount         = $TDT.Scripts.Count()
           Show-Log -LogType Txt -LogText "  Running Scripts.Count       : $ScriptsCount"
           $ScriptErrorCount     = $Script.ErrorCount
           Show-Log -LogType Txt -LogText "  Running Script.ErrorCount   : $ScriptErrorCount"
           $ScriptExecutionTime  = $Script.ExecutionTime
           Show-Log -LogType Txt -LogText "  Running Script.ExecutionTime: $ScriptExecutionTime"
           $ScriptOutputText     = $Script.OutputText
           Show-Log -LogType Txt -LogText "  Running Script.OutputText   : $ScriptOutputText" 
           Show-LogSeperator
         

           # Delete the Login object at index 0
           Show-Log -LogType Txt -LogText "Running Scripts.Delete"
           $TDT.Scripts.Delete(0)


           Show-Log -LogType Txt -LogText 'Execute Script Complete'

          }
       else 
          {Write-Host 'Invalid Connection'}
      }

   }

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }
