﻿#Requires -Version 3.0

#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: Uninstall.ps1 
# Description:
#    Uninstalls demo schema from Source and Target databases and 
#    resets database configurations previously set in Config.ps1           
#-----------------------------------------------------------------


# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Config.ps1') 
Import-Module (Join-Path $PSScriptRoot '..\PSModules\LogUtils.psm1') -Force


# Start Transcript of DEMO Schema Install
$ErrorActionPreference = "SilentlyContinue"
Stop-Transcript        | out-null
$ErrorActionPreference = "Continue"
$LogRunDate     = Get-Date -Format yyyyMMddHHmmss
$LogFile        = 'OracleDemoSchemaUninstall_Log.txt'
$LogPath        = "$LogDir\$LogFile"
Start-Transcript -path $LogPath -append


# Locate SQL*Plus
try {
  $SQLPlusBinary = (Get-Command -Name sqlplus.exe -ErrorAction Stop).Path
} catch {
  Show-Log -LogType Txt -LogText 'SQLPlus.exe could not be located. Install SQL*Plus and add its path to PATH environment variable, execute Install.sql from Toad, etc.'
  exit 1
}


$StartDateTime = Get-Date 
Show-Log -LogType Hdr -LogText 'Oracle DEMO Schema Uninstall'

# Prompt for source DB info
Show-Log -LogType Txt -LogText 'Getting Source Database Login Information...'
Show-Log -LogType Txt -LogText "Source Database Name: $SourceDatabase"
$SourceUser           = Read-Host 'Enter Source Database Username (Requires SYSDBA privs)'
$SourcePasswordSecure = Read-Host -AsSecureString 'Enter Source Database Password'
$SourcePassword       = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SourcePasswordSecure))



Write-Output ''

# Prompt for target DB info
Show-Log -LogType Txt -LogText 'Getting Target Database Login Information...'
Show-Log -LogType Txt -LogText "Target Database Name: $TargetDatabase"
$TargetUser           = Read-Host 'Enter Target Database Username (Requires SYSDBA privs)'
$TargetPasswordSecure = Read-Host -AsSecureString 'Enter Target Database Password'
$TargetPassword       = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($TargetPasswordSecure))


Write-Output ''


# Confirm removal of DEMO schema
    $Choice = ""
    while ($Choice -NotMatch "[Y|N]") 
       {$Confirmation = "DEMO schema will be removed from $SourceDatabase and $TargetDatabase. Continue [Y/N]?"
        $Choice = Read-Host $Confirmation
       }
    
    if ($Choice -eq "N") 
       {exit 1}

    # Setup script params
    $Params = @(
    $SourceDatabase
    $SourceUser
    $SourcePassword
    $TargetDatabase
    $TargetUser
    $TargetPassword)

    # Run install.sql
    Show-Log -LogType Txt -LogText 'Running Uninstall.sql'
    $Script = "$OracleDir\Uninstall.sql"
    & $SQLPlusBinary -l $SourceUser/$SourcePassword@$SourceDatabase as SYSDBA @$Script $Params 2>&1

    if ($LastExitCode) 
       {Show-Log -LogType Txt -LogText 'Warning!!! Install.sql failed'
        exit 1
       }


    # Add Source and Target Database Names to Config.ps1 script
    Show-LogSeperator
    Show-Log -LogType Txt -LogText "Updating Config.ps1 script..."
    Show-Log -LogType Txt -LogText "    SourceDatabase = {SourceDBName}"
    Show-Log -LogType Txt -LogText "    TargetDatabase = {TargetDBName}"
    Show-LogSeperator

    $FileText = Get-Content "$PSScriptsDir\Config.ps1" -Raw
    $FileText = $FileText -Replace "(?ms)(^.*?\`$SourceDatabase\b[^']+')([^']+)(.*)", "`$1{SourceDBName}`$3"
    $FileText = $FileText -Replace "(?ms)(^.*?\`$TargetDatabase\b[^']+')([^']+)(.*)", "`$1{TargetDBName}`$3"
    $FileText | Out-File -FilePath "$PSScriptsDir\Config.ps1" -Force


Show-Log -LogType End -LogText $StartDateTime
Stop-Transcript
Invoke-Item $LogPath
