﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: UnitTesting.ps1 
# Description:
#    To demonstrate various Unit Testing calls. 
#-----------------------------------------------------------------


# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


# Check to see if Install has been run before - if it hasn't, run Install.ps1
if ($SourceDatabase -eq '{SourceDBName}')
   {Show-Log -LogType Txt -LogText 'DEMO schema not found.  Running Install...'
    & "$PSScriptsDir\Install.ps1"
    
    if ($LastExitCode) 
       {exit 1}
    else
       {. (Join-Path $PSScriptRoot '.\Config.ps1')}  #Rerun Config.ps1 after changes were made.
   }


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Unit Testing Log'
   $TDT = $null
   $TDT = New-Object -ComObject $TDTObjectName    # Start TDT
 
   # Check if TDT was started
   if ($TDT)
      {#-- Make Connections to Source and Target
       $Source = $TDT.Connections.NewConnection($UnitTestingSourceConnection)
       $Target = $TDT.Connections.NewConnection($UnitTestingTargetConnection)    

       
       #  Assign Unit Testing to Variable
       $UnitTesting            = $TDT.UnitTesting       
       
       Show-Log -LogType Txt -LogText "Setting Main Parameters from Config File Parameters"

       $UnitTesting.ImportOptions.IncludeSourceCode = $UnitTestingImportOptionsIncludeSourceCode
       $UnitTesting.ImportOptions.IncludeRunResults = $UnitTestingImportOptionsIncludeRunResults
       $UnitTesting.ImportOptions.TestOwner         = $UnitTestingImportOptionsTestOwner
       $UnitTesting.ImportOptions.ObjectOwner       = $UnitTestingImportOptionsObjectOwner

       # Add DB objects
       Show-Log -LogType Txt -LogText "Running Unitesting.UnitTests.DBObjects.Add(): $UnitTestingUnitTestsDBObjectsObjectName"
       $DBObject = $UnitTesting.UnitTests.DBObjects.Add()
       $DBObject.ObjectOwner = $UnitTestingUnitTestsDBObjectsObjectOwner
       $DBObject.ObjectName  = $UnitTestingUnitTestsDBObjectsObjectName

       # Assign Source connection to UnitTesting
       $UnitTesting.Connection = $Source


       #-------------------------------------------------------------
       # Unit Test and Test Suite Imports into Source
       # Note:  These are previously saved xml files specifically for this demo.
       #        This is needed so Export from source works correctly.
       #        The files will be overwritten in the Export from Source.
       #-------------------------------------------------------------
       Show-Log -LogType Txt -LogText "Running Import in $UnitTestingSourceConnection"   

       # Assign Target connection to UnitTesting
       $UnitTesting.Connection = $Source
       
       # Unit Test Import
       $XMLFile = Get-ChildItem "$UnitTestsDir\Input - Tests" 

       foreach($File in $XMLFile)
         {$XML = Get-Content "$UnitTestsDir\Input - Tests\$File" -Raw
          $result = $UnitTesting.Import($XML)

          if ($result)
             {Show-Log -LogType Txt -LogText "  Import ($File)     : Complete"}
          else
             {Show-Log -LogType Txt -LogText "  Import ($File)     : Failed"}
         }
 
 
       # Test Suite Import
       $XMLFile = Get-ChildItem "$UnitTestsDir\Input - Suites" 

       foreach($Suite in $XMLFile)
         {$XML = Get-Content "$UnitTestsDir\Input - Suites\$Suite" -Raw
          $result = $UnitTesting.Import($XML)

          if ($result)
             {Show-Log -LogType Txt -LogText "  Import ($Suite): Complete"}
          else
             {Show-Log -LogType Txt -LogText "  Import ($Suite): Failed"}
         }

       #-------------------------------------------------------------
       # Unit Test and Test Suite Exports from Source
       #-------------------------------------------------------------

       Show-LogSeperator
       Show-Log -LogType Txt -LogText "Running Export from $UnitTestingSourceConnection"   
   
              
       # Get all unit tests for the object(s)
       $UnitTesting.UnitTests.Refresh()


       # Export Tests from Source Database
       $i = 0
       foreach ($UnitTest in $UnitTesting.UnitTests) 
          {$i++
           $Filename     = "UnitTesting_$i.xml"
           $UnitTestFile = "$UnitTestsDir\Output - Tests\$Filename"
           $UnitTest.ExportOptions.IncludeSourceCode = $UnitTestingExportOptionsIncludeSourceCode
           $UnitTest.ExportOptions.IncludeRunResults = $UnitTestingExportOptionsIncludeRunResults
           $UnitTest.ExportOptions.IncludeTimestamps = $UnitTestingExportOptionsIncludeTimestamps  
           # NOTE! UTF8 Encoding is required else import using CTO GUI will fail
           $UnitTest.Export() | Out-File -FilePath $UnitTestFile -Encoding 'UTF8' -Force   
           Show-Log -LogType Txt -LogText "  Export ($Filename)     : Complete"
          } 


       # Get all Unit Test Suites for the object(s)
       $UnitTesting.TestSuites.Refresh()      


       # Export TestSuites from Source Database
       $i = 0
       foreach ($Suite in $UnitTesting.TestSuites) 
          {$i++
           $Filename     = "UnitTesting_Suite$i.xml"
           $UnitTestFile = "$UnitTestsDir\Output - Suites\$Filename"
           $Suite.ExportOptions.IncludeSourceCode = $UnitTestingExportOptionsIncludeSourceCode
           $Suite.ExportOptions.IncludeRunResults = $UnitTestingExportOptionsIncludeRunResults
           $Suite.ExportOptions.IncludeTimestamps = $UnitTestingExportOptionsIncludeTimestamps   
           # NOTE! UTF8 Encoding is required else import using CTO GUI will fail
           $Suite.Export() | Out-File -FilePath $UnitTestFile -Encoding 'UTF8' -Force
           Show-Log -LogType Txt -LogText "  Export ($Filename): Complete"   
          }


       #-------------------------------------------------------------
       # Unit Test and Test Suite Imports
       #-------------------------------------------------------------
       Show-LogSeperator
       Show-Log -LogType Txt -LogText "Running Import in $UnitTestingTargetConnection"   

       # Assign Target connection to UnitTesting
       $UnitTesting.Connection = $Target
       
       # Unit Test Import
       $XMLFile = Get-ChildItem "$UnitTestsDir\Output - Tests" 

       foreach($File in $XMLFile)
         {$XML = Get-Content "$UnitTestsDir\Output - Tests\$File" -Raw
          $result = $UnitTesting.Import($XML)

          if ($result)
             {Show-Log -LogType Txt -LogText "  Import ($File)     : Complete"}
          else
             {Show-Log -LogType Txt -LogText "  Import ($File)     : Failed"}
         }
 
 
       # Test Suite Import
       $XMLFile = Get-ChildItem "$UnitTestsDir\Output - Suites" 

       foreach($Suite in $XMLFile)
         {$XML = Get-Content "$UnitTestsDir\Output - Suites\$Suite" -Raw
          $result = $UnitTesting.Import($XML)

          if ($result)
             {Show-Log -LogType Txt -LogText "  Import ($Suite): Complete"}
          else
             {Show-Log -LogType Txt -LogText "  Import ($Suite): Failed"}
         }
       
       
       
       #-------------------------------------------------------------
       # Unit Test and Test Suite Runs
       #-------------------------------------------------------------

       Show-LogSeperator
       Show-Log -LogType Txt -LogText "Running Tests in $UnitTestingTargetConnection"   


       # Running UnitTests
       Show-Log -LogType Txt -LogText "Running Unitesting.UnitTests.Execute()"
       $Res = $UnitTesting.UnitTests.Execute()
  
       if ($Res) 
          {Write-Output '  Run Unit Tests : Passed'} 
       else 
          {Write-Output '  Run Unit Tests : Failed'}


       # Running Unit Test Suites
       Show-Log -LogType Txt -LogText "Running Unitesting.TestSuites.Execute()"
       $Res = $UnitTesting.TestSuites.Execute()
  
       if ($Res) 
          {Write-Output '  Run Test Suites: Passed'} 
       else 
          {Write-Output '  Run Test Suites: Failed'}

      
   }

}

catch{$ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"
      
      throw
     }


finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }

