﻿#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: ValidationChecks.ps1 
# Description:
#    To verify TDT is installed and can be called.  Also, verifies
#    TDT can connect to the Source and Target Databases setup during
#    the demo installation.
#-------------------------------------------------------------

# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Initialize.ps1')


# Check to see if Install has been run before - if it hasn't, run Install.ps1
if ($SourceDatabase -eq '{SourceDBName}')
   {Show-Log -LogType Txt -LogText 'DEMO schema not found.  Running Install...'
    & "$PSScriptsDir\Install.ps1"
    
    if ($LastExitCode) 
       {exit 1}
    else
       {. (Join-Path $PSScriptRoot '.\Config.ps1')}  #Rerun Config.ps1 after changes were made.
   }


try
  {# Display Log Headers
   $StartDateTime = Get-Date 
   Show-Log -LogType Hdr -LogText 'Validation Checks'

   # Check if TDT Object Exists
   $Object = Find-Object -ObjectName $TDTObjectName

   if ($Object) 
      {Show-Log -LogType Txt -LogText "Verifying $TDTObjectName Exists: Success"
                                         
  
       #Start TDT
       $TDT = New-Object -ComObject $TDTObjectName


       if ($TDT)
          {Show-Log -LogType Txt -LogText  "Starting  $TDTObjectName       : Success"}
       else
          {Show-Log -LogType Txt -LogText  "Starting  $TDTObjectName       : Failed"
           throw
          }

        Show-LogSeperator

        
        # Connect to Source database
        $DBConnection = $TDT.Connections.NewConnection($SourceConnection)

        $Connected = $DBConnection.Connected

        if ($Connected)
           {$DBVersion        = $DBConnection.Version
            $DBVersionVerbose = $DBConnection.VersionVerbose
            Show-Log -LogType Txt -LogText "Connection to DB $SourceConnection : Success"           
            Show-Log -LogType Txt -LogText "   Database Version        : $DBVersion"
            Show-Log -LogType Txt -LogText "   Database Version Verbose: $DBVersionVerbose"
            $DBConnection.Disconnect()             
            $Connected = $DBConnection.Connected          
            if ($Connected)
               {Show-Log -LogType Txt -LogText "   Disconnect : Failed"}
            else
               {Show-Log -LogType Txt -LogText "   Disconnect : Success"}
           }
        else
           {Show-Log -LogType Txt -LogText "Connection to $SourceConnection : Failed"
            throw
           }



        # Connect to Target database
        $DBConnection = $TDT.Connections.NewConnection($TargetConnection)

        $Connected = $DBConnection.Connected

        if ($Connected)
           {$DBVersion        = $DBConnection.Version
            $DBVersionVerbose = $DBConnection.VersionVerbose
            Show-Log -LogType Txt -LogText "Connection to DB $TargetConnection : Success"           
            Show-Log -LogType Txt -LogText "   Database Version        : $DBVersion"
            Show-Log -LogType Txt -LogText "   Database Version Verbose: $DBVersionVerbose"
            $DBConnection.Disconnect()             
            $Connected = $DBConnection.Connected          
            if ($Connected)
               {Show-Log -LogType Txt -LogText "   Disconnect : Failed"}
            else
               {Show-Log -LogType Txt -LogText "   Disconnect : Success"}          
           }
        else
           {Show-Log -LogType Txt -LogText "Connection to $TargetConnection : Failed"
            throw
           }


        Show-LogSeperator

           
        Show-Log -LogType Txt -LogText "End of Validation Checks"     
      }
   else
      {Show-Log -LogType Txt -LogText "Verifying $TDTObjectName Exists : Failed"
       throw
      }

   }

catch{# TDT Specific Debug Log Information
      $TDT.Debug.Logfile.GetText()               | Out-File -FilePath "$DebugDir\Logfile.txt"       -Force
      $TDT.Debug.ExceptionLog.GetLastError()     | Out-File -FilePath "$DebugDir\LastError.txt"     -Force
      $TDT.Debug.ExceptionLog.GetLastCallStack() | Out-File -FilePath "$DebugDir\LastCallStack.txt" -Force
      $TDT.Debug.ExceptionLog.GetLastReport()    | Out-File -FilePath "$DebugDir\LastReport.txt"    -Force
      $TDT.Debug.ExceptionLog.GetFullReport()    | Out-File -FilePath "$DebugDir\FullReport.txt"    -Force

      # PowerShell Specific Debug Log Information
      $ErrorMessage  = $_.Exception.Message
      $ErrorLineNum  = $_.InvocationInfo.ScriptLineNumber
      $ErrorPosition = $_.InvocationInfo.OffsetInLine
      $ErrorLine     = $_.InvocationInfo.Line

      Write-Output "************ ERROR ************"
      Write-Output "Line Number/Pos: $ErrorLineNum : $ErrorPosition"
      Write-Output "Error Message  : $ErrorMessage"
      Write-Output "Error Line     : $ErrorLine"
      Write-Output "*******************************"

      if ($TDT)
         {$TDT.Quit()} 

     throw   
     }

finally {# Run Cleanup Processes
         . (Join-Path $PSScriptRoot '.\Finalize.ps1')
        }